<?php

namespace IPS\chatbox\api;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Chat API
 */
class _messages extends \IPS\Content\Api\ItemController
{
	/**
	 * Class
	 */
	protected $class = 'IPS\chatbox\Rooms\Chat';
	
	/**
	 * GET /chatbox/messages
	 * Get list of messages
	 *
	 * @note		For requests using an OAuth Access Token for a particular member, only messages the authorized user can view will be included
	 * @apiparam	string	room			room ID
	 * @apiparam	string	sortBy			What to sort by. Can be 'date' for creation date, 'title' or leave unspecified for ID
	 * @apiparam	string	sortDir			Sort direction. Can be 'asc' or 'desc' - defaults to 'asc'
	 * @apiparam	int		page			Page number
	 * @apiparam	int		perPage			Number of results per page - defaults to 25
	 * @return		\IPS\Api\PaginatedResponse<IPS\chatbox\Rooms\Chat>
	 */
	public function GETindex()
	{
		/* Where clause */
		$where = array( array('chat_sys IS NULL') );
				
		/* Return */
		return $this->_list( $where, 'room' );
	}
	
	/**
	 * GET /chatbox/messages/{id}
	 * View information about a specific message
	 *
	 * @param		int		$id				ID Number
	 * @throws		1BIMCB111/1	INVALID_ID		The message ID does not exist or the authorized user does not have permission to view it
	 * @return		\IPS\chatbox\Rooms\Chat
	 */
	public function GETitem( $id )
	{
		try
		{
			$chat = \IPS\chatbox\Rooms\Chat::load( $id );
			// if (  !$chat->container()->_canView() )
			// {
				// throw new \OutOfRangeException;
			// }
			return new \IPS\Api\Response( 200, $chat->apiOutput( $this->member ) );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', '1BIMCB111/1', 404 );
		}
	}

	/**
	 * POST /chatbox/messages/
	 * Send message
	 *
	 * @reqapiparam	int						room			The ID number of the room
	 * @reqapiparam	int						author			The ID number of the user
	 * @reqapiparam	string					content			The message content
	 * @throws		1BIMCB111/7				NO_ROOM			The room ID does not exist
	 * @throws		1BIMCB111/8				NO_USER			The user ID does not exist
	 * @throws		1BIMCB111/9				NO_MESSAGE		No message content was supplied
	 * @throws		1BIMCB111/H				NO_PERMISSION		No permission to chat
	 * @throws		1BIMCB111/10			ERROR_MESSAGE_MAX_LENGHT	Error message: Message character limit reached
	 * @throws		1BIMCB111/11			ERROR_MESSAGE_LINKS			Error message: contain black list URL
	 * @throws		1BIMCB111/12			ERROR_MESSAGE_BAD_CHARS		Error message: too many special characters
	 * @return		\IPS\chatbox\Rooms\Chat
	 */
	public function POSTindex()
	{
		/* Get room */
		try
		{
			$room = \IPS\chatbox\Rooms\Room::load( \IPS\Request::i()->room );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'NO_CATEGEORY', '1BIMCB111/7', 400 );
		}
		
		/* Get author */
		if ( ! \IPS\Request::i()->author ) 
		{
			throw new \IPS\Api\Exception( 'NO_AUTHOR', '1BIMCB111/8', 400 );
		}
		
		$author = \IPS\Member::load( \IPS\Request::i()->author );
		if ( !$author->member_id )
		{
			throw new \IPS\Api\Exception( 'NO_AUTHOR', '1BIMCB111/8', 400 );
		}

		/* Check permission */
		if ( ! $room->canChat( $author ) )
		{
			throw new \IPS\Api\Exception( 'NO_PERMISSION', '1BIMCB111/H', 403 );
		}	
		
		/* Check we have a message content */
		if ( !\IPS\Request::i()->content )
		{
			throw new \IPS\Api\Exception( 'NO_MESSAGE', '1BIMCB111/9', 400 );
		}
					
		/* Add message */
		$opts = $room->options();
		$msg = \IPS\Application::load('chatbox')->checkMSG( \IPS\Request::i()->content, $opts, true );
		
		$values['chatbox_room'] = $room->id;
		$values['chatMSG_' . $room->id] = $msg;		
		$chat = \IPS\chatbox\Rooms\Chat::createItem( $author, NULL, \IPS\DateTime::create(), $room );
		$chat->processForm( $values );
		$chat->member_id = $author->member_id;
		$chat->save();

		$room->cacheLastMsg( $chat->id, $chat->time );
		
		/* Done */
		return new \IPS\Api\Response( 201, $chat->apiOutput( $this->member ) );
	}
	
	/**
	 * POST /chatbox/messages/{id}
	 * Edit a message
	 *
	 * @note	For requests using an OAuth Access Token for a particular member, any parameters the user doesn't have permission to use are ignored (for example, locked will only be honoured if the authenticated user has permission to lock records).
	 * @apiparam	string				content			New message content
	 * @throws		1BIMCB111/1			INVALID_ID		The message ID is invalid or the authorized user does not have permission to view it
	 * @throws		1BIMCB111/9				NO_MESSAGE		No message content was supplied
	 * @throws		1BIMCB111/14			NO_PERMISSION		No permission to edit
	 * @throws		1BIMCB111/10			ERROR_MESSAGE_MAX_LENGHT	Error message: Message character limit reached
	 * @throws		1BIMCB111/11			ERROR_MESSAGE_LINKS			Error message: contain black list URL
	 * @throws		1BIMCB111/12			ERROR_MESSAGE_BAD_CHARS		Error message: too many special characters
	 * @return		\IPS\chatbox\Rooms\Chat
	 */
	public function POSTitem( $id )
	{
		/* Get message */
		try
		{
			$chat = \IPS\chatbox\Rooms\Chat::load( $id );
			
			return new \IPS\Api\Response( 200, $chat->apiOutput( $this->member ) );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', '1BIMCB111/1', 404 );
		}
		
		/* Check we have a message content */
		if ( !\IPS\Request::i()->content )
		{
			throw new \IPS\Api\Exception( 'NO_MESSAGE', '1BIMCB111/9', 400 );
		}
		
		$opts = $chat->container()->options();
		$content = \IPS\Application::load('chatbox')->checkMSG( \IPS\Request::i()->content, $opts, true );
		$chat->content = $content;
		$chat->title = \IPS\Application::load('chatbox')->autoTitle( $content ? $content : \IPS\Member::loggedIn()->language()->get('chatbox_txt_img') );
		$chat->title_furl = \IPS\Http\Url\Friendly::seoTitle( $this->title );
		$chat->save();
		
		/* Done */
		return new \IPS\Api\Response( 201, $chat->apiOutput( $this->member ) );		
	}
		
	/**
	 * DELETE /chatbox/messages/{id}
	 * Delete a message
	 *
	 * @param		int		$id			ID Number
	 * @throws		1BIMCB111/1	INVALID_ID	The message ID does not exist
	 * @return		void
	 */
	public function DELETEitem( $id )
	{
		try
		{
			$chat = \IPS\chatbox\Rooms\Chat::load( $id );

			$chat->delete();
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', '1BIMCB111/1', 404 );
		}
	}
}